
#ifndef RAIL_SDK
#define RAIL_SDK
#include "flexsimbasicdefs.h"
// uncomment the following to be compatible with 4.3

// ranks of a rail move node created by addrailmove()
#define MOVE_TRAVELERS 1
#define MOVE_KINEMATIC 2
#define MOVE_START_TIME 3
#define MOVE_TRAIN_SIZE 4
#define MOVE_DEALLOCATE_ID 5
#define MOVE_PATH 6
#define MOVE_COUPLE_DECOUPLE_OFFSET 7
#define MOVE_COUPLE_DECOUPLE_AFTER 8

// ranks of a path structure
#define PATH_COUPLINGS 1
#define PATH_FIRST_EDGE 2
#define PATH_FIRST_DIST 3
#define PATH_TOTAL_DIST 4
#define PATH_FORWARD 5
#define PATH_EDGE_LIST 6

//ranks of a rail send message node created by addrailsendmessage()
#define MESSAGE_TOOBJ 1
#define MESSAGE_FROMOBJ 2
#define MESSAGE_P1 3
#define MESSAGE_P2 4
#define MESSAGE_P3 5
#define MESSAGE_DELAY_TIME 6

// ranks of a rail sequence node created by createrailsequence()
#define SEQUENCE_TRAVELERS 1
#define SEQUENCE_TASKS 2
#define SEQUENCE_TRAIN_LENGTH 3

// ranks of the rail network manager's labels
#define MANAGER_SEQUENCES 1
#define MANAGER_EDGETABLE 2
#define MANAGER_PATHS 3
#define MANAGER_PATHLISTENERS 4
#define MANAGER_SHOWPATHS 5
#define MANAGER_SHOWSWITCHES 6
#define MANAGER_SHOWALLOCATIONS 7
#define MANAGER_SHOWVELOCITIES 8
#define MANAGER_SHOWTURNOUTS 9
#define MANAGER_UPDATE_MODE 10
#define MANAGER_UPDATE_INTERVAL 11
#define MANAGER_NEXT_UPDATE_EVENT 12

#define MANAGER_UPDATE_MODE_DISCRETE 0
#define MANAGER_UPDATE_MODE_TICK 1

#define EVENT_LOOP_STATE_IDLE 0
#define EVENT_LOOP_STATE_ACTIVE 1

// messages sent to the rail network manager
#define MANAGER_FINISH_SEQUENCE_ITEM 1
#define MANAGER_DEALLOCATE_EDGE 2
#define MANAGER_DEALLOCATE_FINAL_EDGE 3
#define MANAGER_ONPATHLISTEN 4
#define MANAGER_UPDATE_EVENT_LOOP 5

// different types of rail sequence operations. Either a move, message, delay or wait
#define SEQUENCE_MOVE 1
#define SEQUENCE_MESSAGE 2
#define SEQUENCE_DELAY 3
#define SEQUENCE_WAIT 4

// column numbers of the rail network manager's edge table
#define EDGETABLE_FROM 1
#define EDGETABLE_TO 2
#define EDGETABLE_DIST 3
#define EDGETABLE_TRACK_TYPE 4
#define EDGETABLE_TRACK_NUMBER 5
#define EDGETABLE_OUTPORT 6
#define EDGETABLE_OPOPNO 7
#define EDGETABLE_NAME 8
#define EDGETABLE_ALLOCATIONS 9
#define EDGETABLE_LISTENERS 10
#define EDGETABLE_USER_DATA 11
#define EDGETABLE_LAST_COLUMN 11

#define NETNODE_SWITCH 1
#define NETNODE_SWITCH_MAXSPEED 2
#define NETNODE_SWITCH_DELAYTIME 3

//Network Node labels
#define NETNODE_LABEL_SWITCH_MODE						1
#define NETNODE_LABEL_SWITCH_THROW_TIME					2
#define NETNODE_LABEL_TURNOUT_SWITCH_MODE				3
#define NETNODE_LABEL_TURNOUT_MAX_SPEED					4
#define NETNODE_LABEL_SWITCH_ID							5
#define NETNODE_LABEL_TOTAL_SWITCH_THROWS				6

// rail traveler data
#define RAIL_TRAVELER_POINTER 1
#define RAIL_TRAVELER_OFFSET 2
#define RAIL_TRAVELER_ACTIVE 3

// rail move couple/decouple traveler data
#define RAIL_MOVE_TRAVELER_POINTER 1
#define RAIL_MOVE_TRAVELER_OFFSET 2
#define RAIL_MOVE_TRAVELER_COUPLE 3


#define EDGE_START 1
#define EDGE_END 2

class linkedlist;
typedef linkedlist fsnode;

// The following are some quick accessors to the network's member table, distance table, and nextnode table
treenode nndisttable();

treenode nnnextnodetable();
treenode nnmembers();

int nnmodified();

// quick accessor the rail network manager
treenode railnetworkmanager();

// getnextnode is quick access to the network's next node table. 
// Pass in member rank from and member rank to, and it gives you the output port (or edge num) of the node to get to the next node.
int getnextnode(int fromnoderank, int tonoderank);

// getnetnodemember is quick access to the network's member table. Pass in the rank, and it will give you back the actual object.
treenode getnetnodemember(int noderank);

int getnetnodeport(int edge, int startorend);

void resetrailglobalpointers();
#endif